@@ -7,8 +7,7 @@
     SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
-#ifndef __LIBKLEOPATRACLIENT_GUI_CERTIFICATEREQUESTER_H__
-#define __LIBKLEOPATRACLIENT_GUI_CERTIFICATEREQUESTER_H__
+#pragma once
 
 #include "kleopatraclientgui_export.h"
 
@@ -31,7 +30,7 @@ class KLEOPATRACLIENTGUI_EXPORT CertificateRequester : public QWidget
     Q_PROPERTY(QStringList selectedCertificates READ selectedCertificates WRITE setSelectedCertificates)
 public:
     explicit CertificateRequester(QWidget *parent = nullptr, Qt::WindowFlags f = {});
-    ~CertificateRequester();
+    ~CertificateRequester() override;
 
     void setMultipleCertificatesAllowed(bool allow);
     bool multipleCertificatesAllowed() const;
@@ -70,4 +69,3 @@ private:
 }
 }
 
-#endif /* __LIBKLEOPATRACLIENT_GUI_CERTIFICATEREQUESTER_H__ */
