@@ -7,8 +7,7 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
-#ifndef __KLEOPATRA_UTILS_FILEDIALOG_H__
-#define __KLEOPATRA_UTILS_FILEDIALOG_H__
+#pragma once
 
 #include <QString>
 #include <QWidget>
@@ -26,5 +25,4 @@ QString getSaveFileNameEx(QWidget *parent = nullptr, const QString &caption = QS
 }
 }
 
-#endif // __KLEOPATRA_UTILS_FILEDIALOG_H__
 
